package meerkat.registry.AsyncRegistryCallbacks;

import com.google.common.util.concurrent.FutureCallback;
import javassist.NotFoundException;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessage;
import meerkat.protobuf.VoterRegistry;

import java.io.IOException;
import java.util.List;

import static meerkat.util.BulletinBoardUtils.getLatestMessage;

// TODO : add logging

/**
 * Created by Vladimir Eliezer on 3/12/2016.
 * AsyncRegistry getGroups creates this callback to list of groups the voter in
 */
public class GetGroupsCallback implements FutureCallback<List<BulletinBoardMessage>> {
    protected FutureCallback<List<VoterRegistry.GroupID>> callback;

    public GetGroupsCallback(FutureCallback<List<VoterRegistry.GroupID>> callback){
        this.callback = callback;
    }

    /**
     * Gets List of GroupId objects from given message
     * @param message   BulletinBoardMessage which contains all groups ids of voter
     * @return list of GroupId
     * @throws IOException
     * @throws ClassNotFoundException
     */
    private List<VoterRegistry.GroupID> getVoterGroups(BulletinBoardMessage message) throws IOException, ClassNotFoundException {
        VoterRegistry.VoterRegistryMessage voterRegistryMessage =
                VoterRegistry.VoterRegistryMessage.parseFrom(message.getMsg().getData());

        return voterRegistryMessage.getGroupIDList();
    }


    @Override
    public void onSuccess(List<BulletinBoardMessage> messages) {
        if (messages.isEmpty()){
            callback.onFailure(new NotFoundException("Voter permissions not found in database"));
            return;
        }

        BulletinBoardMessage message = getLatestMessage(messages);
        try {
            callback.onSuccess(getVoterGroups(message));
        } catch (IOException | ClassNotFoundException e) {
            callback.onFailure(e);
        }
    }

    @Override
    public void onFailure(Throwable t) {
        this.callback.onFailure(t);
    }
}
