package meerkat.registry.AsyncRegistryCallbacks;

import com.google.common.util.concurrent.FutureCallback;
import com.google.protobuf.InvalidProtocolBufferException;
import javassist.NotFoundException;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessage;
import meerkat.protobuf.VoterRegistry;

import java.util.List;

import static meerkat.util.BulletinBoardUtils.getLatestMessage;

// TODO : add logging

/**
 * Created by Vladimir Eliezer  on 3/12/2016.
 * AsyncRegistry getVoter creates this callback to get voterInfo protobuff
 */
public class GetVoterCallback implements FutureCallback<List<BulletinBoardMessage>> {
    protected FutureCallback<VoterRegistry.VoterInfo> callback;

    public GetVoterCallback(FutureCallback<VoterRegistry.VoterInfo> callback){
        this.callback = callback;
    }

    @Override
    public void onSuccess(List<BulletinBoardMessage> messages) {
        if(messages.isEmpty()){
            callback.onFailure(new NotFoundException("Voter info not found in database."));
            return;
        }
        BulletinBoardMessage latestMessage = getLatestMessage(messages);

        try {
            VoterRegistry.VoterInfo info = VoterRegistry.VoterInfo.newBuilder()
                    .mergeFrom(latestMessage.getMsg().getData()).build();
            callback.onSuccess(info);
        } catch (InvalidProtocolBufferException e) {
            callback.onFailure(e);
        }
    }

    @Override
    public void onFailure(Throwable t) {
        callback.onFailure(t);
    }
}
